@@ -1,13 +1,13 @@
 /* mpf_sub -- Subtract two floats.
 
-Copyright 1993, 1994, 1995, 1996, 1999, 2000, 2001, 2002, 2004 Free Software
-Foundation, Inc.
+Copyright 1993, 1994, 1995, 1996, 1999, 2000, 2001, 2002, 2004, 2005 Free
+Software Foundation, Inc.
 
 This file is part of the GNU MP Library.
 
 The GNU MP Library is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
-the Free Software Foundation; either version 2.1 of the License, or (at your
+the Free Software Foundation; either version 3 of the License, or (at your
 option) any later version.
 
 The GNU MP Library is distributed in the hope that it will be useful, but
@@ -16,9 +16,7 @@ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
 License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
-along with the GNU MP Library; see the file COPYING.LIB.  If not, write to
-the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
-MA 02111-1307, USA. */
+along with the GNU MP Library.  If not, see http://www.gnu.org/licenses/.  */
 
 #include "gmp.h"
 #include "gmp-impl.h"
@@ -33,7 +31,7 @@ mpf_sub (mpf_ptr r, mpf_srcptr u, mpf_srcptr v)
   mp_exp_t exp;
   mp_size_t ediff;
   int negate;
-  TMP_DECL (marker);
+  TMP_DECL;
 
   usize = u->_mp_size;
   vsize = v->_mp_size;
@@ -62,7 +60,7 @@ mpf_sub (mpf_ptr r, mpf_srcptr u, mpf_srcptr v)
       return;
     }
 
-  TMP_MARK (marker);
+  TMP_MARK;
 
   /* Signs are now known to be the same.  */
   negate = usize < 0;
@@ -196,7 +194,7 @@ mpf_sub (mpf_ptr r, mpf_srcptr u, mpf_srcptr v)
 	  vsize = prec - 1;
 	}
 
-      tp = (mp_ptr) TMP_ALLOC (prec * BYTES_PER_MP_LIMB);
+      tp = TMP_ALLOC_LIMBS (prec);
       {
 	mp_limb_t cy_limb;
 	if (vsize == 0)
@@ -277,7 +275,7 @@ general_case:
 
   /* Allocate temp space for the result.  Allocate
      just vsize + ediff later???  */
-  tp = (mp_ptr) TMP_ALLOC (prec * BYTES_PER_MP_LIMB);
+  tp = TMP_ALLOC_LIMBS (prec);
 
   if (ediff >= prec)
     {
@@ -407,5 +405,5 @@ general_case:
   if (rsize == 0)
     exp = 0;
   r->_mp_exp = exp;
-  TMP_FREE (marker);
+  TMP_FREE;
 }
