@@ -1,6 +1,6 @@
 /* mpz_realloc2 -- change allocated data size.
 
-Copyright 2001, 2002 Free Software Foundation, Inc.
+Copyright 2001, 2002, 2008 Free Software Foundation, Inc.
 
 This file is part of the GNU MP Library.
 
@@ -17,22 +17,29 @@ License for more details.
 You should have received a copy of the GNU Lesser General Public License
 along with the GNU MP Library.  If not, see http://www.gnu.org/licenses/.  */
 
+#include <stdlib.h>
+#include <stdio.h>
 #include "gmp.h"
 #include "gmp-impl.h"
 
 void
-mpz_realloc2 (mpz_ptr m, unsigned long bits)
+mpz_realloc2 (mpz_ptr m, mp_bitcnt_t bits)
 {
-  mp_ptr mp;
   mp_size_t new_alloc;
 
-  new_alloc = (bits + GMP_NUMB_BITS-1) / GMP_NUMB_BITS;
+  bits -= (bits != 0);		/* Round down, except if 0 */
+  new_alloc = 1 + bits / GMP_NUMB_BITS;
 
-  /* Never allocate zero space. */
-  new_alloc = MAX (new_alloc, 1);
+  if (sizeof (unsigned long) > sizeof (int)) /* param vs _mp_size field */
+    {
+      if (UNLIKELY (new_alloc > INT_MAX))
+	{
+	  fprintf (stderr, "gmp: overflow in mpz type\n");
+	  abort ();
+	}
+    }
 
-  mp = __GMP_REALLOCATE_FUNC_LIMBS (PTR(m), ALLOC(m), new_alloc);
-  PTR(m) = mp;
+  PTR(m) = __GMP_REALLOCATE_FUNC_LIMBS (PTR(m), ALLOC(m), new_alloc);
   ALLOC(m) = new_alloc;
 
   /* Don't create an invalid number; if the current value doesn't fit after
