@@ -6,7 +6,7 @@ This file is part of the GNU MP Library.
 
 The GNU MP Library is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
-the Free Software Foundation; either version 2.1 of the License, or (at your
+the Free Software Foundation; either version 3 of the License, or (at your
 option) any later version.
 
 The GNU MP Library is distributed in the hope that it will be useful, but
@@ -15,9 +15,7 @@ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
 License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
-along with the GNU MP Library; see the file COPYING.LIB.  If not, write to
-the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-MA 02110-1301, USA. */
+along with the GNU MP Library.  If not, see http://www.gnu.org/licenses/.  */
 
 #include "gmp.h"
 #include "gmp-impl.h"
@@ -29,14 +27,14 @@ MA 02110-1301, USA. */
    (except when in the high limb), but usually the search won't go very far
    so it seems reasonable to inline that code.  */
 
-unsigned long
-mpz_scan1 (mpz_srcptr u, unsigned long starting_bit)
+mp_bitcnt_t
+mpz_scan1 (mpz_srcptr u, mp_bitcnt_t starting_bit)
 {
   mp_srcptr      u_ptr = PTR(u);
   mp_size_t      size = SIZ(u);
   mp_size_t      abs_size = ABS(size);
   mp_srcptr      u_end = u_ptr + abs_size;
-  unsigned long  starting_limb = starting_bit / GMP_NUMB_BITS;
+  mp_size_t      starting_limb = starting_bit / GMP_NUMB_BITS;
   mp_srcptr      p = u_ptr + starting_limb;
   mp_limb_t      limb;
   int            cnt;
@@ -124,7 +122,7 @@ mpz_scan1 (mpz_srcptr u, unsigned long starting_bit)
         {
           p++;
           if (p == u_end)
-            return abs_size * GMP_NUMB_BITS;
+            return (mp_bitcnt_t) abs_size * GMP_NUMB_BITS;
           limb = *p;
         }
 
@@ -135,5 +133,5 @@ mpz_scan1 (mpz_srcptr u, unsigned long starting_bit)
  got_limb:
   ASSERT (limb != 0);
   count_trailing_zeros (cnt, limb);
-  return (p - u_ptr) * GMP_NUMB_BITS + cnt;
+  return (mp_bitcnt_t) (p - u_ptr) * GMP_NUMB_BITS + cnt;
 }
